﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.getCentroid = getCentroid;

    getCentroid.implemented = {
        BoundingBox: doBoundingBox,
        Point: doPoint,
        MultiPoint: doMultiPoint,
        LineString: doLineString,
        MultiLineString: doMultiLineString,
        Polygon: doPolygon,
        MultiPolygon: doMultiPolygon,
        GeometryCollection: doGeometryCollection
    };

    return ns;

    //---------------------------------------------------------

    function getCentroid(geometry) {

        var type = GJ.getType(geometry);
        var func = getCentroid.implemented[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "getCentroid");

        return func(geometry);
    }

    //---------------------------------------------------------

    function doBoundingBox(bbox) {
        return GJ.createPoint([
            (bbox[0] + bbox[2]) / 2,
            (bbox[1] + bbox[3]) / 2,
        ]);
    }

    function doPoint(geometry) {
        return GJ.createPoint(geometry.coordinates);
    }

    function doMultiPoint(geometry) {
        var bbox = GJ.getBoundingBox(geometry);
        doBoundingBox(bbox);
    }

    function doLineString(geometry) {
        var length = GJ.getLength(geometry);
        var ftr = GJ.getFeatureAt(geometry, length / 2);

        if (ftr)
            return ftr.geometry;
        else
            return null;
    }

    function doMultiLineString(geometry) {
        var longest = GJ.getLongest(geometry);
        return doLineString(longest);
    }

    function doPolygon(geometry) {

        var dx = Number.MAX_VALUE;
        var dy = Number.MAX_VALUE;

        geometry.coordinates[0].forEach(function (coord) {
            if (coord[0] < dx) dx = coord[0];
            if (coord[1] < dy) dy = coord[1];
        });
        
        var paths = [];
        geometry.coordinates[0].forEach(function (coord) {
            paths.push([coord[0] - dx, coord[1] - dy]);
        });

        var X = 0;
        var Y = 0;
        var AA = 0;
        for (var i = 0; i < paths.length - 1; i++) {
            X += (paths[i][0] + paths[i + 1][0]) * (paths[i][0] * paths[i + 1][1] - paths[i + 1][0] * paths[i][1]);

            Y += (paths[i][1] + paths[i + 1][1]) * (paths[i][0] * paths[i + 1][1] - paths[i + 1][0] * paths[i][1]);

            AA += (paths[i][0] * paths[i + 1][1] - paths[i + 1][0] * paths[i][1]);
        }
        var A = AA * 0.5;
        var x = 1 / (6 * A) * X;
        var y = 1 / (6 * A) * Y;

        return GJ.createPoint([x + dx, y + dy]);
    }

    function doMultiPolygon(geometry) {

        var totalArea = 0;
        var res = [];

        geometry.coordinates.forEach(function (coord) {

            var poly = {
                type: "Polygon",
                coordinates: coord
            };

            var centroid = doPolygon(poly).coordinates;
            var area = GJ.getArea(poly);

            if (totalArea == 0) {
                res = centroid;
                totalArea = area;
            }
            else {
                res[0] = (totalArea * res[0] + area * centroid[0]) / (totalArea + area);
                res[1] = (totalArea * res[1] + area * centroid[1]) / (totalArea + area);
                totalArea += area;
            }
        });

        return GJ.createPoint(res);
    }

    function doGeometryCollection(geometry) {
        var bbox = GJ.getBoundingBox(geometry);
        doBoundingBox(bbox);
    }

})(Mapzania || {});