﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.Leaflet = ns.Leaflet || {};
    ns.Leaflet.View = View;
    return ns;

    //---------------------------------------------------------

    function View(elementId) {

        var view = this;
        var GJ = ns.GeoJson;

        ns.Util.eventify(this, "Leaflet.View");

        validate();

        var map = L.map(elementId, { zoomControl: false });

        addControls();
        addEvents();

        var layers = new ns.Leaflet.Layers(map);
        var labelLayer = layers.createLabelLayer();
        var selectorLayer = layers.createSelectorLayer();

        //---------------------- STARTUP --------------------------

        function validate() {
            if (typeof L == "undefined")
                ns.Util.logAndThrow("Mapzania cannot continue because Leaflet is not installed.", "Leaflet.View");
            else
                ns.Log.debug("Object created.", "Leaflet.View");
        }

        function addControls() {
            view.toolbar = new ns.Leaflet.Toolbar();
            map.addControl(view.toolbar);

            //var legend = L.control.layers(null, null, { collapsed: false }).addTo(map);
            L.control.scale({ imperial: false }).addTo(map);
        }

        function addEvents() {
            map.on("moveend", function (event) {
                view.fire("changed", view.getState());
            });

            map.on("click", function (e) {
                view.fire("clicked", {
                    position: { type: "Point", coordinates: [e.latlng.lng, e.latlng.lat] },
                    pixel: { x: e.layerPoint.x, y: e.layerPoint.y }
                });
            })

            map.on("dblclick", function (e) {
                view.fire("double_clicked", {
                    position: { type: "Point", coordinates: [e.latlng.lng, e.latlng.lat] },
                    pixel: { x: e.layerPoint.x, y: e.layerPoint.y }
                });
            })
        }

        //---------------------- FACTORY --------------------------

        this.getPixelTransformer = function () {
            return new ns.Leaflet.PixelTransformer(map);
        }

        //------------------------ MAP ----------------------------

        this.getState = function () {
            var bounds = ns.Leaflet.Convert.latLngBoundsToBBox(map.getBounds());
            var pixelBounds = ns.Leaflet.Convert.pixelBoundsToBBox(map.getPixelBounds());
            var center = map.getCenter();

            return {
                bounds: bounds,
                pixelBounds: pixelBounds,
                zoom: map.getZoom(),
                center: { latitude: center.lat, longitude: center.lng }
            };
        }

        this.fitToBoundingBox = function (bbox) {
            map.fitBounds([[bbox[1], bbox[0]], [bbox[3], bbox[2]]]);
        }

        this.zoomIn = function () {
            map.zoomIn(1);
        }

        this.zoomOut = function () {
            map.zoomOut(1);
        }

        this.setMouseMode = function (mode) {

            selectorLayer.clearLayers();

            switch (mode) {

                case ns.MouseModes.Pan:
                    map.doubleClickZoom.enable();
                    document.getElementById(elementId).style.cursor = "-webkit-grab";
                    break;

                case ns.MouseModes.Query: //TODO
                    break;

                case ns.MouseModes.DrawPoint:
                case ns.MouseModes.DrawLine:
                case ns.MouseModes.DrawPolygon:
                    map.doubleClickZoom.disable();
                    document.getElementById(elementId).style.cursor = "crosshair";
                    break;
            }
        }

        //----------------------LAYERS-----------------------------

        this.addLayer = function (layer, index) {

            switch (layer.getType()) {
                case "RasterLayer":
                    layers.addRasterLayer(layer, index);
                    break;
                case "VectorLayer":
                    layers.addVectorLayer(layer, index);
                    break;
                default:
            }
        }

        this.showLayer = function (layer) {
            layers.get(layer.getKey()).show();
        }

        this.hideLayer = function (layer) {
            layers.get(layer.getKey()).hide();
        }

        this.removeLayer = function (layer) {
            layers.remove(layer.getKey());
        }

        this.sendLayerBackwards = function (layer) {
            layers.get(layer.getKey()).sendBackwards();
        }

        this.bringLayerForwards = function (layer) {
            layers.get(layer.getKey()).bringForwards();
        }

        this.replaceLayer = function (key, layer) {
            layers.remove(key);
            this.addLayer(layer);
        }

        this.drawLayer = function (layerKey, data) {
            var layer = layers.get(layerKey);
            layer.clear();
            layer.add(data);
        }

        this.drawLabels = function (data) {
            labelLayer.clearLayers();
            labelLayer.addLabels(data);
        }

        this.drawSelector = function (feature) {

            view.toDraw = feature;

            //hack to allow dblclick event to propogate
            setTimeout(function () {

                if (!view.toDraw) {
                    selectorLayer.clearLayers();
                    return;
                }

                if (feature.type == "LineString") {
                    view.toDraw = { type: "FeatureCollection", features: [] };
                    view.toDraw.features.push(feature);

                    for (var i = 0; i < feature.coordinates.length; i++) {
                        view.toDraw.features.push(GJ.createPoint(feature.coordinates[i]));
                    }
                }

                if (feature.type == "Polygon") {
                    view.toDraw = { type: "FeatureCollection", features: [] };
                    view.toDraw.features.push(feature);

                    for (var i = 0; i < feature.coordinates[0].length; i++) {
                        view.toDraw.features.push(GJ.createPoint(feature.coordinates[0][i]));
                    }
                }

                selectorLayer.clearLayers();
                selectorLayer.addData(view.toDraw);
            }, 100);


        }

        this.setBackColor = function (color) {
            document.getElementById(elementId).style.backgroundColor = color;
        }

        //---------------------- TOOLBAR --------------------------

        this.Toolbar = {
            addButton: function (button) {
                view.toolbar.addButton(button);
            }
        };
    }

})(Mapzania || {});

