﻿(function () {
    'use strict';

    angular
        .module('app')
        .service('CommandService', service);

    service.$inject = ["Application"];

    //-------------------------------------------------

    function service(Application) {

        var svc = this;
        svc.commands = new CommandManager();

        svc.reset = reset;

        svc.copyBoundsFromMap = doCopyBoundsFromMap;

        svc.change = doChange;
        svc.changeAndUpdate = doChangeAndUpdate;
        svc.changeBackColor = doChangeBackColor;

        svc.sendLayerBackwards = doSendLayerBackwards;
        svc.bringLayerForwards = doBringLayerForwards;
        svc.addLayer = doAddLayer;
        svc.removeLayer = doRemoveLayer;
        svc.changeLayerVisibility = doChangeLayerVisibility;
        svc.changeLayerGreyScale = doChangeLayerGreyScale

        svc.addTheme = doAddTheme;
        svc.removeTheme = doRemoveTheme;

        svc.addThemeValue = doAddThemeValue;
        svc.removeThemeValue = doRemoveThemeValue;
        svc.autoPopulateThemeValues = doAutoPopulateThemeValues;

        svc.undo = undo;
        svc.redo = redo;
        svc.clean = clean;

        svc.isDirty = isDirty;
        svc.canUndo = canUndo;
        svc.canRedo = canRedo;

        svc._executed = function () {
            return svc.commands.executed.map(m => m.name);
        }

        svc._undone = function () {
            return svc.commands.undone.map(m => m.name);
        }

        //---------------------------------------------

        function reset() {
            svc.commands.reset();
        }

        //---------------------------------------------

        function doCopyBoundsFromMap() {

            var oldVal = Application.model.getStartExtent();
            var newVal = Object.assign({}, oldVal);

            var bounds = Application.view.getBoundingBox();
            newVal.minX = bounds[0];
            newVal.minY = bounds[1];
            newVal.maxX = bounds[2];
            newVal.maxY = bounds[3];

            function fn(data) {
                Application.model.getMap().startExtent = data.newVal;
            }

            function undo(data) {
                Application.model.getMap().startExtent = data.oldVal;
            }

            var cmd = new Command("CopyBoundsFromMap",
                fn,
                undo,
                Application.model.getStartExtent,
                Application.model.getStartExtent,
                {
                    newVal: newVal,
                    oldVal: oldVal
                }
            );

            svc.commands.execute(cmd);
        }

        function doChange(appliesTo, prop, newVal, oldVal) {

            function fn(data) {
                appliesTo[data.prop] = data.newVal;
            }

            function undo(data) {
                appliesTo[data.prop] = data.oldVal;
            }

            var cmd = new Command("Change",
                fn,
                undo,
                () => appliesTo,
                () => appliesTo,
                {
                    prop: prop,
                    newVal: newVal,
                    oldVal: oldVal
                }
            );

            svc.commands.execute(cmd);
        }

        function doChangeAndUpdate(appliesTo, prop, newVal, oldVal, layerKey) {

            function fn(data) {
                appliesTo[data.prop] = data.newVal;
                Application.view.updateLayer(layerKey);
            }

            function undo(data) {
                appliesTo[data.prop] = data.oldVal;
                Application.view.updateLayer(layerKey);
            }

            var cmd = new Command("ChangeAndUpdate",
                fn,
                undo,
                () => appliesTo,
                () => appliesTo,
                {
                    prop: prop,
                    newVal: newVal,
                    oldVal: oldVal
                }
            );

            svc.commands.execute(cmd);
        }

        function doChangeLayerVisibility(layer, newVal, oldVal) {

            function fn(data) {
                layer.visible = data.newVal;

                if (layer.visible)
                    Application.view.showLayer(layer.key);
                else
                    Application.view.hideLayer(layer.key);
            }

            function undo(data) {
                layer.visible = data.oldVal;

                if (layer.visible)
                    Application.view.showLayer(layer.key);
                else
                    Application.view.hideLayer(layer.key);
            }

            var cmd = new Command("ChangeLayerVisibility",
                fn,
                undo,
                () => layer,
                () => layer,
                {
                    newVal: newVal,
                    oldVal: oldVal
                }
            );

            svc.commands.execute(cmd);
        }

        function doChangeLayerGreyScale(layer, newVal, oldVal) {

            function fn(data) {
                layer.greyScale = data.newVal;
                Application.view.replaceLayer(layer.key, layer);
            }

            function undo(data) {
                layer.greyScale = data.oldVal;
                Application.view.replaceLayer(layer.key, layer);
            }

            var cmd = new Command("ChangeLayerGreyScale",
                fn,
                undo,
                () => layer,
                () => layer,
                {
                    newVal: newVal,
                    oldVal: oldVal
                }
            );

            svc.commands.execute(cmd);
        }

        function doChangeBackColor(newVal, oldVal) {

            function fn(data) {
                Application.model.getMap().backColor = data.newVal;
                Application.view.setBackgroundColor(data.newVal);
            }

            function undo(data) {
                Application.model.getMap().backColor = data.oldVal;
                Application.view.setBackgroundColor(data.oldVal);
            }

            var cmd = new Command("ChangeBackColor",
                fn,
                undo,
                Application.model.getMap,
                Application.model.getMap,
                {
                    newVal: newVal,
                    oldVal: oldVal
                }
            );

            svc.commands.execute(cmd);
        }

        function doSendLayerBackwards(layer) {

            var cmd = new Command("SendLayerBackwards",
                sendLayerBackwards,
                bringLayerForwards,
                () => layer,
                () => layer,
                layer
            );

            svc.commands.execute(cmd);
        }

        function doBringLayerForwards(layer) {

            var cmd = new Command("BringLayerForwards",
                bringLayerForwards,
                sendLayerBackwards,
                () => layer,
                () => layer,
                layer
            );

            svc.commands.execute(cmd);
        }

        function doAddLayer(layer) {

            var data = {
                layer: layer,
                index: 0
            };

            var cmd = new Command("AddLayer",
                addLayer,
                removeLayer,
                () => layer,
                Application.model.getMap,
                data
            );

            svc.commands.execute(cmd);
        }

        function doRemoveLayer(layer) {

            var index = Application.model.indexOfLayer(layer);

            var data = {
                layer: layer,
                index: index
            };

            var cmd = new Command("RemoveLayer",
                removeLayer,
                addLayer,
                Application.model.getMap,
                () => layer,
                data);

            svc.commands.execute(cmd);
        }

        function doAddTheme(layer, theme) {

            var index = layer.themes.length;

            var data = {
                index: index,
                layer: layer,
                theme: theme
            };

            var cmd = new Command("AddTheme",
                addTheme,
                removeTheme,
                () => theme,
                () => layer,
                data
            );

            svc.commands.execute(cmd);
        }

        function doRemoveTheme(layer, theme) {

            var index = layer.themes.indexOf(theme);

            var data = {
                index: index,
                layer: layer,
                theme: theme
            };

            var cmd = new Command("AddTheme",
                removeTheme,
                addTheme,
                () => layer,
                () => theme,
                data
            );

            svc.commands.execute(cmd);
        }

        function doAddThemeValue(layer, theme, themeValue) {

            var index = theme.values.length;

            var data = {
                index: index,
                layer:layer,
                theme: theme,
                themeValue: themeValue
            };

            var cmd = new Command("AddThemeValue",
                addThemeValue,
                removeThemeValue,
                () => theme,
                () => theme,
                data
            );

            svc.commands.execute(cmd);
        }

        function doRemoveThemeValue(layer, theme, themeValue) {

            var index = theme.values.indexOf(themeValue);

            var data = {
                index: index,
                layer: layer,
                theme: theme,
                themeValue: themeValue
            };

            var cmd = new Command("RemoveThemeValue",
                removeThemeValue,
                addThemeValue,
                () => theme,
                () => theme,
                data
            );

            svc.commands.execute(cmd);
        }

        function doAutoPopulateThemeValues(theme) {
        }

        //---------------------------------------------

        function sendLayerBackwards(layer) {
            Application.model.sendLayerBackwards(layer);
            Application.view.sendLayerBackwards(layer.key);
        }

        function bringLayerForwards(layer) {
            Application.model.bringLayerForwards(layer);
            Application.view.bringLayerForwards(layer.key);
        }

        function addLayer(data) {
            Application.model.addLayer(data.layer, data.index);
            Application.view.addLayer(data.layer.key, data.layer, data.index);
            Application.view.updateLayer(data.layer.key);
        }

        function removeLayer(data) {
            Application.model.removeLayer(data.layer);
            Application.view.removeLayer(data.layer.key);
        }

        function addTheme(data) {
            Application.model.addTheme(data.layer, data.theme, data.index);
            Application.view.addTheme(data.layer.key, data.theme);
            Application.view.updateLayer(data.layer.key);
        }

        function removeTheme(data) {
            Application.model.removeTheme(data.layer, data.theme);
            Application.view.removeTheme(data.layer.key, data.index);
            Application.view.updateLayer(data.layer.key);
        }

        function addThemeValue(data) {
            var themeIndex = data.layer.themes.indexOf(data.theme);

            Application.model.addThemeValue(data.theme, data.themeValue, data.index);
            Application.view.updateTheme(data.layer.key, themeIndex, data.theme);
            Application.view.updateLayer(data.layer.key);
        }

        function removeThemeValue(data) {
            var themeIndex = data.layer.themes.indexOf(data.theme);

            Application.model.removeThemeValue(data.theme, data.themeValue, data.index);
            Application.view.updateTheme(data.layer.key, themeIndex, data.theme);
            Application.view.updateLayer(data.layer.key);
        }

        //---------------------------------------------

        function undo() {
            return svc.commands.undo();
        }

        function redo() {
            return svc.commands.redo();
        }

        function clean() {
            var list = svc.commands.executed;
            svc.lastCommand = list[list.length - 1];
        }

        function isDirty() {
            var list = svc.commands.executed;
            return svc.lastCommand != list[list.length - 1];
        }

        function canUndo() {
            return svc.commands.executed.length > 0;
        }

        function canRedo() {
            return svc.commands.undone.length > 0;
        }

        //---------------------------------------------

        function CommandManager() {

            this.executed = [];
            this.undone = [];

            this.reset = function () {
                this.executed = [];
                this.undone = [];
            }

            this.execute = function execute(cmd) {
                cmd.execute(cmd.data);
                this.executed.push(cmd);
                this.undone = [];
                console.log("Cmd executed...", cmd);
            };

            this.undo = function undo() {
                if (this.executed.length == 0)
                    return null;

                var cmd = this.executed.pop();
                cmd.undo(cmd.data);

                this.undone.push(cmd);

                console.log("Cmd undone...", cmd);

                return cmd.undoResult();
            };

            this.redo = function redo() {

                if (this.undone.length == 0)
                    return null;

                var cmd = this.undone.pop();

                cmd.execute(cmd.data);
                this.executed.push(cmd);

                console.log("Cmd redone...", cmd);

                return cmd.redoResult();
            };
        }

        function Command(name, execute, undo, redoResult, undoResult, data) {
            this.name = name,
            this.execute = execute;
            this.undo = undo;
            this.redoResult = redoResult;
            this.undoResult = undoResult;
            this.data = data;
        }
    }

})();